package graph;

import java.util.Collection;
import java.util.HashSet;

/**
 * Represents the set of vertices that are in conflict with a certain vertex.
 * 
 * @author sepphar
 * 
 * @param <V>
 */
public class ConflictSet<V> extends HashSet<V> implements Comparable<ConflictSet<V>>
{
	private static final long serialVersionUID = 1L;

	public static <V, E> ConflictSet<V> copy(ConflictSet<V> conflict)
	{
		if ( conflict != null )
			return new ConflictSet<V>(conflict.v, conflict);
		return null;
	}

	private final V v;

	public ConflictSet(V v, Collection<V> conflictVertices)
	{
		super();
		if ( conflictVertices != null )
		{
			this.addAll(conflictVertices);
		}
		this.v = v;
	}

	@Override
	public int compareTo(ConflictSet<V> o)
	{
		return this.size() - o.size();
	}

	public V getV()
	{
		return this.v;
	}

	@Override
	public String toString()
	{
		return "(" + this.v + ">" + super.toString() + ")";
	}

}