package solvers;

import graph.GraphAnalysis;

import java.io.IOException;
import java.util.Collection;
import java.util.Set;

import org.jgrapht.Graph;

import benchmark.helpClasses.LogFileWriter;

/**
 * 
 * Finds the largest 2-club in a graph. The parameter k will be ignored.
 * 
 * @author SeppHar
 * 
 */
public class OptimisationSolver<V, E> extends Solver<V, E>
{

	public OptimisationSolver(Graph<V, E> graph, Collection<V> requiredVertices)
	{
		super(graph, -1, requiredVertices);
	}

	@Override
	public Set<V> solve() throws UnsolvableException
	{
		if ( Solver.verbose )
		{
			try
			{
				GraphAnalysis.analyseGraph(new LogFileWriter(), this.getGraph(), false);
				System.out.println();
			} catch (final IOException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		Set<V> twoClub = GraphAnalysis.getMaxDegreeTwoClub(this.getGraph()).getS2();
		int solutionSize = twoClub.size() + 1;

		if ( Solver.verbose )
		{
			System.out.println("k:" + solutionSize);
		}

		while (true)
		{

			final Solver<V, E> solver = new DualSearchTreeSolver<V, E>(this.getGraph(), solutionSize);
			Set<V> tmp;
			if ( (tmp = solver.solve()) != null )
			{
				solutionSize = tmp.size() + 1;
				twoClub = tmp;
			}
			else
				return twoClub;
		}
	}

}
