package solvers;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

import org.jgrapht.Graph;

public class TaiwanSolver<V, E> extends Solver<V, E>
{

	private Graph<V, E> g;

	float time;

	// public static void main(String[] args) throws UnsolvableException
	// {
	// final int noVertices = 150;
	// final float a = 0.1f;
	// final float b = 0.2f;
	//
	// final SimpleGraph<StdVertex, DefaultEdge> graph =
	// GraphAnalysis.createPGeneratedGraph(noVertices, a, b);
	// TaiwanSolver<StdVertex, DefaultEdge> s = new TaiwanSolver<StdVertex, DefaultEdge>(graph);
	// s.solve();
	//
	// }

	public TaiwanSolver(Graph<V, E> graph)
	{
		super(graph, -1, null);
	}

	public float getTime()
	{
		return this.time;
	}

	// @Override
	@Override
	public Set<V> solve() throws UnsolvableException
	{
		final String fileName = "graph.des";
		Set<V> twoClub = null;

		try
		{
			int i = 0;
			final HashMap<V, Integer> vertexToIntMap = new HashMap<V, Integer>();
			final HashMap<Integer, V> intToVertexMap = new HashMap<Integer, V>();
			for (final V v : this.getGraph().vertexSet())
			{
				vertexToIntMap.put(v, i);
				intToVertexMap.put(i++, v);
			}

			this.writeGraph(fileName, vertexToIntMap);

			String line;
			final String command = "./kclub " + fileName + " 2";
			final Process process = Runtime.getRuntime().exec(command);
			final Reader r = new InputStreamReader(process.getInputStream());
			final BufferedReader in = new BufferedReader(r);

			while ((line = in.readLine()) != null)
			{
				final String timeKey = "total time:";
				final String sClubKey = "2club";
				if ( line.startsWith(timeKey) )
				{
					final String substring = line.substring(line.indexOf(timeKey) + timeKey.length());
					this.time = Float.parseFloat(substring);
				}
				else if ( line.startsWith(sClubKey) )
				{
					twoClub = new HashSet<V>();
					String substring = line.substring(line.indexOf(sClubKey) + sClubKey.length() + 1);
					substring = substring.trim();
					final String[] arr = substring.split(" ");
					for (final String s : arr)
					{
						twoClub.add(intToVertexMap.get(Integer.parseInt(s)));
					}
				}
			}
			in.close();

			new File(fileName).delete();

		} catch (final IOException e)
		{
			e.printStackTrace();
		}
		return twoClub;
	}

	private void writeGraph(String fileName, HashMap<V, Integer> vertexToIntMap) throws IOException
	{
		final int n = this.getGraph().vertexSet().size();
		final int m = this.getGraph().edgeSet().size();

		final File f = new File(fileName);
		final FileWriter writer = new FileWriter(f);
		writer.write(n + " " + m + "\n");

		for (final E e : this.getGraph().edgeSet())
		{
			writer.write(vertexToIntMap.get(this.getGraph().getEdgeSource(e)) + " "
					+ vertexToIntMap.get(this.getGraph().getEdgeTarget(e)) + "\n");
		}
		writer.close();

	}
}
