package solvers.dataReduction;

import graph.ConflictCachingGraph;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import solvers.Solver;

public class AdvancedRuleProvider<V, E> extends SimpleRuleProvider<V, E>
{

	protected final ConflictCachingGraph<V, E> g;

	public AdvancedRuleProvider(Solver<V, E> solver, ConflictCachingGraph<V, E> graph)
	{
		super(solver, graph);
		this.g = graph;
	}

	public Map<V, Set<V>> getConflictMapping()
	{
		return this.g.getConflictMapping();
	}

	public ConflictCachingGraph<V, E> getGraph()
	{
		return this.g;
	}

	@Override
	public int getNoCommonNeighbors(V v1, V v2)
	{
		return this.g.getNoOfCommonNeigbors(v1, v2);
	}

	@Override
	public Set<V> getTwoNeigh(V v)
	{
		if ( this.getTwoNeighSize(v) * 2 > this.vertexSet().size() )
		{
			final Set<V> neigh = new HashSet<V>();
			final Set<V> conflict = this.g.getConflictMapping().get(v);
			for (final V n : this.g.vertexSet())
			{
				if ( !conflict.contains(n) )
				{
					neigh.add(n);
				}
			}
			return neigh;
		}
		return super.getTwoNeigh(v);
	}

	@Override
	public int getTwoNeighSize(V v)
	{
		return this.vertexSet().size() - (this.g.getConflictMapping().get(v).size());
	}
}