package solvers.dataReduction;

import graph.ConflictCachingGraph;

import java.util.HashSet;
import java.util.Stack;

import solvers.DirtyAddVertex;
import solvers.DualSearchTreeSolver;
import solvers.Solver;
import solvers.UnsolvableException;

/**
 * Remembers all the changes that have been done to an instance and enables you to roll them back.
 * 
 * @author sepphar
 * 
 * @param <V>
 * @param <E>
 */
public class ChangeLogger<V, E> extends AdvancedRuleProvider<V, E>
{
	private final Stack<DirtyAddVertex<V>> removedVertices;

	private HashSet<V> requiredVertices;

	public ChangeLogger(DualSearchTreeSolver<V, E> solver)
	{
		this(solver, solver.getBranchingGraph());
	}

	public ChangeLogger(Solver<V, E> solver, ConflictCachingGraph<V, E> graph)
	{
		super(solver, graph);
		this.removedVertices = new Stack<DirtyAddVertex<V>>();
		this.requiredVertices = new HashSet<V>();
	}

	@Override
	public void removeVertex(V v) throws UnsolvableException
	{
		if ( this.getRequiredVertices().contains(v) || this.g.vertexSet().size() <= this.getTwoClubMinSize() )
			throw new UnsolvableException();
		this.removedVertices.push(this.g.removeDirtyVertex(v));
	}

	public void reverseAllChanges()
	{
		while (!this.removedVertices.isEmpty())
		{
			this.g.addVertex(this.removedVertices.pop());
		}
		this.solver.getRequiredVertices().removeAll(this.requiredVertices);
	}

	@Override
	public void setVertexRequired(V v)
	{
		this.requiredVertices.add(v);
		super.setVertexRequired(v);
	}
}