package solvers.dataReduction;

import java.util.Iterator;
import java.util.Set;

import solvers.TuringKernelSolver;
import solvers.UnsolvableException;
import dataStructures.BinaryHeap;
import dataStructures.ComparableStore;

public class FastHighDegreeRule<V, E> implements RuleInterface<V, E>
{
	final int maxQueueSize = TuringKernelSolver.upperBound * 10;

	class CompStoreVerticesByDegree extends ComparableStore<V, Integer>
	{
		public CompStoreVerticesByDegree(V v, Integer s2)
		{
			super(v, s2);
		}

		@Override
		public boolean equals(Object obj)
		{
			if ( obj instanceof FastHighDegreeRule.CompStoreVerticesByDegree )
				return this.hashCode() == obj.hashCode();
			return super.equals(obj);
		}

		@Override
		public int hashCode()
		{
			return this.getS1().hashCode();
		}

	}

	@Override
	public boolean reduce(SimpleRuleProvider<V, E> provider) throws UnsolvableException
	{
		final BinaryHeap<CompStoreVerticesByDegree> heap = new BinaryHeap<CompStoreVerticesByDegree>(maxQueueSize);

		boolean updateExistingValuesInQueue = true;
		while ((!heap.isEmpty() && heap.peek().getS2() < provider.getTwoClubMinSize())
				|| (this.refillHeap(provider, heap, updateExistingValuesInQueue) && !heap.isEmpty() && heap.peek().getS2() < provider
						.getTwoClubMinSize()))
		{
			updateExistingValuesInQueue = false;

			final CompStoreVerticesByDegree t = heap.poll();

			final Set<V> neigh = provider.getNeighbors(t.getS1());
			provider.removeVertex(t.getS1());
			this.updateQueueForSomeEntries(provider, heap, neigh.iterator());
		}
		return false;
	}

	/**
	 * Refills the queue. Always returns true;
	 * 
	 * @param provider
	 * @param heap
	 * @param updateExistingEntries TODO
	 * @return
	 */
	private boolean refillHeap(SimpleRuleProvider<V, E> provider, final BinaryHeap<CompStoreVerticesByDegree> heap,
			boolean updateExistingEntries)
	{
		if ( updateExistingEntries )
		{
			class IterComp implements Iterator<V>
			{
				private final Iterator<CompStoreVerticesByDegree> iter;

				public IterComp(Iterator<CompStoreVerticesByDegree> i)
				{
					this.iter = i;
				}

				@Override
				public boolean hasNext()
				{
					return this.iter.hasNext();
				}

				@Override
				public V next()
				{
					return this.iter.next().getS1();

				}

				@Override
				public void remove()
				{

				}

			}
			this.updateQueueForSomeEntries(provider, heap, new IterComp(heap.iterator()));
		}

		if ( heap.size() < maxQueueSize && heap.size() < provider.vertexSet().size() )
		{
			int i = 0;
			for (final V v : provider.vertexSet())
			{
				if ( i++ > maxQueueSize )
				{
					break;
				}
				if ( !heap.containElement(new CompStoreVerticesByDegree(v, 0)) )
				{
					heap.add(new CompStoreVerticesByDegree(v, provider.getTwoNeighSize(v)));
				}
			}
		}
		return true;
	}

	private void updateQueueForSomeEntries(SimpleRuleProvider<V, E> provider, final BinaryHeap<CompStoreVerticesByDegree> heap,
			final Iterator<V> iterator)
	{
		while (iterator.hasNext())
		{
			final V n = iterator.next();
			final CompStoreVerticesByDegree nCompEntry;
			if ( heap.containElement(new CompStoreVerticesByDegree(n, 0))
					&& (nCompEntry = heap.get(new CompStoreVerticesByDegree(n, 0))).getS2() >= provider.getTwoClubMinSize() )
			{
				nCompEntry.setS2(provider.getTwoNeighSize(n));
			}
		}
		heap.ensureHeapProperty();
	}
}