package solvers.dataReduction;

import graph.GraphAnalysis;
import graph.VertexMaskedSubgraph;

import java.util.Set;

import solvers.Solver;
import solvers.UnsolvableException;
import solvers.turingKernels.TwoNeighInterface;

public class SimpleRuleProvider<V, E> implements TwoNeighInterface<V, E>
{
	public static boolean ruleVerbose = false;

	private final VertexMaskedSubgraph<V, E> g;

	protected final Solver<V, E> solver;

	public SimpleRuleProvider(Solver<V, E> solver, VertexMaskedSubgraph<V, E> graph)
	{
		this.solver = solver;
		this.g = graph;
	}

	public boolean containsEdge(V v1, V v2)
	{
		return this.g.containsEdge(v1, v2);
	}

	public boolean containsVertex(V v)
	{
		return this.g.containsVertex(v);
	}

	public Set<V> getCommonNeighbors(V v1, V v2)
	{
		return GraphAnalysis.getCommonNeighbors(this.g, v1, v2);
	}

	public int getDegreeOf(V vertex)
	{
		return this.g.getDegreeOf(vertex);
	}

	public V getMinDegreeVertex()
	{
		return GraphAnalysis.getMinDegreeVertex(this.g);
	}

	public Set<V> getNeighbors(V v)
	{
		return this.g.getNeighborsOf(v);
	}

	public int getNoCommonNeighbors(V v1, V v2)
	{
		return GraphAnalysis.getCommonNeighbors(this.g, v1, v2).size();
	}

	public int getNoVertices()
	{
		return this.g.vertexSet().size();
	}

	public Set<V> getRequiredVertices()
	{
		return this.solver.getRequiredVertices();
	}

	public int getTwoClubMinSize()
	{
		return this.solver.getTwoClubMinSizeParam();
	}

	@Override
	public Set<V> getTwoNeigh(V v)
	{
		return GraphAnalysis.getNeighborsWithinDistanceS(v, this.g, 2);
	}

	@Override
	public int getTwoNeighSize(V v)
	{
		return this.getTwoNeigh(v).size();
	}

	public void removeVertex(V v) throws UnsolvableException
	{
		if ( this.getRequiredVertices().contains(v) || this.g.vertexSet().size() <= this.getTwoClubMinSize() )
			throw new UnsolvableException();
		this.g.removeVertex(v);
	}

	public void setVertexRequired(V v)
	{
		this.solver.getRequiredVertices().add(v);
	}

	public Set<V> vertexSet()
	{
		return this.g.vertexSet();
	}
}
