package solvers.turingKernels;

import graph.GraphAnalysis;
import graph.VertexMaskedSubgraph;

import java.util.HashSet;
import java.util.PriorityQueue;
import java.util.Set;
import java.util.Vector;

import solvers.TuringKernelSolver;
import dataStructures.ComparableStore;
import dataStructures.Store;

public class TuringKernelProviderQueue<V, E> implements TuringKernelProvider<V, E>
{
	private final VertexPriority<V> priorityProvider;

	private final TuringKernelSolver<V, E> solver;

	public final static int maximumNumberOfVerticesToBeConsidered = 500;

	public TuringKernelProviderQueue(TuringKernelSolver<V, E> solver, VertexPriority<V> priorityProvider)
	{
		this.solver = solver;
		this.priorityProvider = priorityProvider;
	}

	@Override
	public Store<V, Set<V>> getNextTuringKernel()
	{
		final VertexMaskedSubgraph<V, E> graph = this.solver.getGraphT();
		while (true)
		{
			final Vector<ComparableStore<V, Integer>> vertices = new Vector<ComparableStore<V, Integer>>(
					TuringKernelProviderQueue.maximumNumberOfVerticesToBeConsidered);
			int i = 0;
			for (final V v : graph.vertexSet())
			{
				if ( i++ > TuringKernelProviderQueue.maximumNumberOfVerticesToBeConsidered )
				{
					break;
				}
				vertices.add(new ComparableStore<V, Integer>(v, this.priorityProvider.getVertexPriority(v)));
			}

			final PriorityQueue<ComparableStore<V, Integer>> que = new PriorityQueue<ComparableStore<V, Integer>>(vertices);
			if ( que.isEmpty() )
				return null;

			while (!que.isEmpty())
			{

				final ComparableStore<V, Integer> l = que.poll();
				final HashSet<V> twoNeighSize = GraphAnalysis.getNeighborsWithinDistanceS(l.getS1(), graph, 2);
				if ( twoNeighSize.size() < this.solver.getTwoClubMinSizeParam() )
				{
					this.solver.getGraph().removeVertex(l.getS1());
				}
				else
					return new Store<V, Set<V>>(l.getS1(), twoNeighSize);
			}
		}
	}

	@Override
	public void removeVertex(V vertex)
	{
		this.solver.getGraph().removeVertex(vertex);
	}
}